@@ -7,7 +7,9 @@ import ctypes
 import os
 import shlex
 import socket
+import struct
 import sys
+import time
 
 import asyncssh
 
@@ -21,7 +23,7 @@ from . import utils
 class MicroSSHServer(asyncssh.SSHServer):
     """Micro SSH Server"""
 
-    welcome = """Welcome to turbo-tunnel micro ssh server\n\n"""
+    welcome = b"""Welcome to turbo-tunnel micro ssh server\n\n"""
 
     def __init__(
         self, listen_address, server_host_keys, username, password=None, public_key=None
@@ -58,6 +60,21 @@ class MicroSSHServer(asyncssh.SSHServer):
     async def connection_requested(self, dest_host, dest_port, orig_host, orig_port):
         return await self._conn.forward_connection(dest_host, dest_port)
 
+    def resize(self, fd, size):
+        utils.logger.info(
+            "[%s] Terminal window resize to (%d, %d)"
+            % (self.__class__.__name__, size[0], size[1])
+        )
+        if sys.platform == "win32":
+            # TODO
+            pass
+        else:
+            import fcntl
+            import termios
+
+            winsize = struct.pack("HHHH", size[1], size[0], 0, 0)
+            fcntl.ioctl(fd, termios.TIOCSWINSZ, winsize)
+
     async def _create_process(self, interactive, command, size):
         if not interactive:
             proc = await asyncio.create_subprocess_shell(
@@ -115,6 +132,7 @@ class MicroSSHServer(asyncssh.SSHServer):
                     except Exception as e:
                         sys.stderr.write(str(e))
                 else:
+                    self.resize(fd, size)
                     proc = utils.Process(pid)
                     stdin = utils.AsyncFileDescriptor(fd)
                     stdout = utils.AsyncFileDescriptor(fd)
@@ -163,29 +181,22 @@ class MicroSSHServer(asyncssh.SSHServer):
                     buffer = task.result()
                 except asyncssh.BreakReceived:
                     return -1
-                except asyncssh.TerminalSizeChanged:
-                    # TODO
+                except asyncssh.TerminalSizeChanged as e:
+                    self.resize(stdout, (e.width, e.height))
                     continue
 
-                if isinstance(buffer, bytes):
-                    try:
-                        buffer = buffer.decode("utf8")
-                    except:
-                        buffer = buffer.decode("gbk")
-
                 if not buffer:
                     return -1
 
                 if index == 0:
-                    if buffer.endswith("\r") and sys.platform == "win32":
-                        buffer += "\n"
+                    if buffer.endswith(b"\r") and sys.platform == "win32":
+                        buffer += b"\n"
 
-                    buffer = buffer.encode()
                     stdin.write(buffer)
                 elif index == 1:
                     process.stdout.write(buffer)
                 else:
-                    buffer = buffer.replace("\r\n", "\r").replace("\r", "\r\n")
+                    buffer = buffer.replace(b"\r\n", b"\r").replace(b"\r", b"\r\n")
                     process.stderr.write(buffer)
 
         return proc.returncode
@@ -221,6 +232,7 @@ class MicroSSHServer(asyncssh.SSHServer):
                 process_factory=lambda process: asyncio.ensure_future(
                     self.handle_process(process)
                 ),
+                encoding=None,
                 line_editor=line_editor,
             )
         except OSError as e:
@@ -268,6 +280,13 @@ class SSHTunnelServer(server.TunnelServer, MicroSSHServer):
                 skey = asyncssh.generate_private_key("ssh-rsa")
                 skey.write_private_key(private_key_path)
                 skey.write_public_key(private_key_path + ".pub")
+        private_key_path = os.path.abspath(private_key_path)
+        if not os.path.isfile(private_key_path):
+            raise ValueError("Private key file %s not exist" % private_key_path)
+        if public_key_path:
+            public_key_path = os.path.abspath(public_key_path)
+            if not os.path.isfile(public_key_path):
+                raise ValueError("Public key file %s not exist" % public_key_path)
 
         MicroSSHServer.__init__(
             self,
@@ -366,13 +385,6 @@ class SSHTunnel(tunnel.Tunnel):
                 ssh_conn.abort()
                 await ssh_conn.wait_closed()
                 return None
-            except Exception:
-                import traceback
-
-                traceback.print_exc()
-                ssh_conn.abort()
-                await ssh_conn.wait_closed()
-                return None
             self.__class__.ssh_conns[key] = ssh_conn
         return self.__class__.ssh_conns[key]
 
@@ -380,23 +392,12 @@ class SSHTunnel(tunnel.Tunnel):
         ssh_conn = await self.create_ssh_conn()
         if not ssh_conn:
             return False
-        this = self
-
-        class SSHTCPSession(asyncssh.SSHTCPSession):
-            def data_received(self, data, datatype):
-                this._buffer += data
-
-            def connection_lost(self, exc):
-                this._closed = True
 
         try:
             self._reader, self._writer = await ssh_conn.open_connection(
                 self._addr, self._port
             )
         except asyncssh.ChannelOpenError as e:
-            import traceback
-
-            traceback.print_exc()
             utils.logger.warn(
                 "[%s] Connect %s:%d over %s failed: %s"
                 % (self.__class__.__name__, self._addr, self._port, self._url, e)
@@ -423,6 +424,17 @@ class SSHTunnel(tunnel.Tunnel):
             self._writer = None
         self._reader = None
 
+    async def fork(self):
+        time0 = time.time()
+        tunnel = self.__class__(self._tunnel, self._url, (self._addr, self._port))
+        if await tunnel.connect():
+            utils.logger.info(
+                "[%s][%.3f] %s fork success"
+                % (self.__class__.__name__, (time.time() - time0), tunnel)
+            )
+            return tunnel
+        return None
+
 
 registry.tunnel_registry.register("ssh", SSHTunnel)
 registry.server_registry.register("ssh", SSHTunnelServer)
