@@ -242,6 +242,11 @@ class IStream(object):
             "%s.%s" % (self.__class__.__name__, inspect.currentframe().f_code.co_name)
         )
 
+    async def fork(self):
+        raise NotImplementedError(
+            "%s.%s" % (self.__class__.__name__, inspect.currentframe().f_code.co_name)
+        )
+
 
 class ConfigError(RuntimeError):
     pass
@@ -286,6 +291,9 @@ class AsyncFileDescriptor(object):
         self._loop.add_reader(self._fd, self.read_callback)
         self._closed = False
 
+    def fileno(self):
+        return self._fd
+
     def close(self):
         self._loop.remove_reader(self._fd)
 
